# -*- coding: utf-8 -*-
"""Transform input to geodis compatible xml."""
from jinja2 import Environment, PackageLoader
from roulier.codec import Encoder
from roulier.exception import InvalidApiInput
from .geodis_api import GeodisApi

GEODIS_ACTIONS = ('demandeImpressionEtiquette',)


class GeodisEncoder(Encoder):
    """Transform input to geodis compatible xml."""

    def encode(self, api_input, action):
        """Transform input to geodis compatible xml."""
        if not (action in GEODIS_ACTIONS):
            raise Exception(
                'action %s not in %s' % (action, ', '.join(GEODIS_ACTIONS)))

        api = GeodisApi()
        if not api.validate(api_input):
            raise InvalidApiInput(
                'Input error : %s' % api.errors(api_input))
        data = api.normalize(api_input)

        data['service']['labelFormat'] = self.lookup_label_format(
            data['service']['labelFormat'])
        data['service']['shippingDate'] = data['service']['shippingDate'].replace('/','') 

        is_test = data['service']['is_test']

        env = Environment(
            loader=PackageLoader('roulier', '/carriers/geodis/templates'),
            extensions=['jinja2.ext.with_', 'jinja2.ext.autoescape'],
            autoescape=True)

        template = env.get_template("geodis_%s.xml" % action)
        return {
            "body": template.render(
                service=data['service'],
                parcel=data['parcel'],
                sender_address=data['from_address'],
                receiver_address=data['to_address']),
            "headers": data['auth'],
            "is_test": is_test
        }

    def api(self):
        api = GeodisApi()
        return api.api_values()

    def lookup_label_format(self, label_format="ZPL"):
        """Get a Geodis compatible format of label.

        args:
            label_format: (str) ZPL or PDF
        return
            a value taken in GEODIS_LABEL_FORMAT
        """
        lookup = {
            'ZPL': 'Z',
            'PDF': 'P',
            'HTML': 'H',
            'XML': 'X',
            'EPL2': 'E',
            'Z': 'Z',
            'P': 'P',
            'H': 'H',
            'X': 'X',
            'E': 'E',
        }
        return lookup.get(label_format.upper(), 'Z')



# -*- coding: utf-8 -*-
"""Implement geodisWS."""
import requests
import email.parser
from lxml import objectify
from jinja2 import Environment, PackageLoader
from roulier.transport import Transport
from roulier.ws_tools import remove_empty_tags
import logging

log = logging.getLogger(__name__)


class GeodisTransport(Transport):
    """Implement Geodis WS communication."""

    GEODIS_WS = "http://espace.geodis.com/geolabel/services/ImpressionEtiquette"  # nopep8
    GEODIS_WS_TEST = "http://espace.recette.geodis.com/geolabel/services/ImpressionEtiquette"  # nopep8
    STATUS_SUCCES = "success"
    STATUS_ERROR = "error"

    def send(self, payload):
        """Call this function.

        Args:
            payload.body: XML in a string
            payload.header : auth
        Return:
            {
                status: STATUS_SUCCES or STATUS_ERROR, (string)
                message: more info about status of result (lxml)
                response: (Requests.response)
                payload: usefull payload (if success) (xml as string)

            }
        """
        body = payload['body']
        headers = payload['headers']
        is_test = payload['is_test']
        soap_message = self.soap_wrap(body, headers)
        log.debug(soap_message)
        response = self.send_request(soap_message, is_test)
        log.info('WS response time %s' % response.elapsed.total_seconds())
        return self.handle_response(response)

    def soap_wrap(self, body, auth):
        """Wrap body in a soap:Enveloppe."""
        env = Environment(
            loader=PackageLoader('roulier', '/carriers/geodis/templates'),
            extensions=['jinja2.ext.with_'])

        template = env.get_template("geodis_soap.xml")
        body_stripped = remove_empty_tags(body)
        header_template = env.get_template("geodis_header.xml")
        header_xml = header_template.render(auth=auth)
        data = template.render(body=body_stripped, header=header_xml)
        return data.encode('utf8')

    def send_request(self, body, is_test):
        """Send body to geodis WS."""
        ws_url = self.GEODIS_WS_TEST if is_test else self.GEODIS_WS
        return requests.post(
            ws_url,
            headers={
                'content-type': 'text/xml',
                'SOAPAction': '<SOAP Action>'
            },
            data=body)

    def handle_500(self, response):
        """Handle reponse in case of ERROR 500 type."""
        # TODO : put a try catch (like wrong server)
        # no need to extract_body shit here
        log.warning('Geodis error 500')
        xml = self.get_parts(response)['start']
        obj = objectify.fromstring(xml)
        message = obj.xpath("//*[local-name() = 'message']")
        if len(message) > 0:
            message = message[0] or obj.xpath('//faultstring')[0]
            id_message = obj.xpath("//*[local-name() = 'code']")[0]
        return {
            "id": obj.xpath('//faultcode')[0],
            "status": self.STATUS_ERROR,
            "messages": [{  # only one error msg is returned by ws
                'id': id_message,
                'message': message,
            }],
            "response": response,
        }

    def handle_200(self, response):
        """
        Handle response type 200 (success).

        It still can be a success or a failure.
        """
        parts = self.get_parts(response)
        xml = parts['start']

        def extract_soap(response_xml):
            obj = objectify.fromstring(response_xml)
            return obj.Body.getchildren()[0]

        payload = extract_soap(xml)
        attachement_id = payload.codeAttachement.text[4:]  # remove cid:
        attachement = parts[attachement_id]
        # payload.infoColis.cab

        return {
            "status": "ok",
            "message": "",
            "payload": payload,
            "attachement": attachement,
            "response": response,
        }

    def handle_response(self, response):
        """Handle response of webservice."""
        if response.status_code == 500:
            return self.handle_500(response)
        elif response.status_code == 200:
            return self.handle_200(response)
        else:
            return {
                "status": "error",
                "messages": [{
                    'id': False,
                    'message': "Unexpected status code from server",
                }],
                "response": response
            }

    def get_parts(self, response):
        head_lines = ''
        for k, v in response.raw.getheaders().iteritems():
            head_lines += str(k)+':'+str(v)+'\n'

        full = head_lines + response.content

        parser = email.parser.Parser()
        decoded_reply = parser.parsestr(full)
        parts = {}
        start = decoded_reply.get_param('start').lstrip('<').rstrip('>')
        i = 0
        for part in decoded_reply.get_payload():
            cid = part.get('content-Id', '').lstrip('<').rstrip('>')
            if (not start or start == cid) and 'start' not in parts:
                parts['start'] = part.get_payload()
            else:
                parts[cid or 'Attachment%d' % i] = part.get_payload()
            i += 1

        return parts



# -*- coding: utf-8 -*-
"""Implement laposteWS."""
import requests
import email.parser
from lxml import objectify, etree
from jinja2 import Environment, PackageLoader
from roulier.transport import Transport
from roulier.ws_tools import remove_empty_tags
import logging

log = logging.getLogger(__name__)


class LaposteTransport(Transport):
    """Implement Laposte WS communication."""

    LAPOSTE_WS = "https://ws.colissimo.fr/sls-ws/SlsServiceWS"
    STATUS_SUCCES = "success"
    STATUS_ERROR = "error"

    def send(self, payload):
        """Call this function.

        Args:
            payload.body: XML in a string
            payload.headers: auth
        Return:
            {
                status: STATUS_SUCCES or STATUS_ERROR, (string)
                message: more info about status of result (lxml)
                response: (Requests.response)
                payload: usefull payload (if success) (xml as string)

            }
        """
        body = payload['body']
        headers = payload['headers']
        soap_message = self.soap_wrap(body, headers)
        log.debug(soap_message)
        response = self.send_request(soap_message)
        log.info('WS response time %s' % response.elapsed.total_seconds())
        return self.handle_response(response)

    def soap_wrap(self, body, headers):
        """Wrap body in a soap:Enveloppe."""
        env = Environment(
            loader=PackageLoader('roulier', '/carriers/laposte/templates'),
            extensions=['jinja2.ext.with_'])

        template = env.get_template("laposte_soap.xml")
        body_stripped = remove_empty_tags(body)
        data = template.render(body=body_stripped)
        return data.encode('utf8')

    def send_request(self, body):
        """Send body to laposte WS."""
        return requests.post(
            self.LAPOSTE_WS,
            headers={'content-type': 'text/xml;charset=UTF-8'},
            data=body)

    def handle_500(self, response):
        """Handle reponse in case of ERROR 500 type."""
        # TODO : put a try catch (like wrong server)
        # no need to extract_body shit here
        log.warning('Laposte error 500')
        obj = objectify.fromstring(response.text)
        return {
            "id": obj.xpath('//faultcode')[0],
            "status": self.STATUS_ERROR,
            "message": obj.xpath('//faultstring')[0],
            "response": response,
            "payload": None
        }

    def handle_200(self, response):
        """
        Handle response type 200 (success).

        It still can be a success or a failure.
        """
        def extract_message(response_xml):
            xml = objectify.fromstring(response_xml)
            messages = xml.xpath('//messages')
            exception = False
            for message in messages:
                mess_type = str(message.type)
                if mess_type.lower() == self.STATUS_ERROR.lower():
                    exception = True
            # dirty serialization
            return {
                "exception": exception,
                "message": messages,
            }

        def extract_payload(response_xml):
            xml = objectify.fromstring(response_xml)
            payload_xml = xml.Body.getchildren()[0]
            return etree.tostring(payload_xml)

        def extract_body(response):
            """Because the answer is mixedpart we need to extract."""
            content_type = response.headers['Content-Type']
            boundary = content_type.split('boundary="')[1].split('";')[0]
            start = content_type.split('start="')[1].split('";')[0]

            between_boundaries = response.text.split("--%s" % boundary)[1]
            after_start = between_boundaries.split(start)[1]
            clean_xml = after_start.strip()  # = trim()
            return clean_xml

        response_xml = extract_body(response)

        message = extract_message(response_xml)

        payload = None

        if message['exception']:
            log.warning('Laposte error 200')
            status = self.STATUS_ERROR
        else:
            status = self.STATUS_SUCCES
            payload = extract_payload(response_xml)
        log.info('status: %s' % status)
        return {
            "status": status,
            "message": message,
            "payload": payload,
            "response": response,
        }

    def handle_response(self, response):
        """Handle response of webservice."""
        if response.status_code == 500:
            return self.handle_500(response)
        elif response.status_code == 200:
            return self.handle_200(response)
        else:
            return {
                "status": "error",
                "message": "Unexpected status code from server",
                "response": response
            }

    def get_parts(self, response):
        head_lines = ''
        for k, v in response.raw.getheaders().iteritems():
            head_lines += str(k) + ':' + str(v) + '\n'

        full = head_lines + response.content

        parser = email.parser.Parser()
        decoded_reply = parser.parsestr(full)
        parts = {}
        start = decoded_reply.get_param('start').lstrip('<').rstrip('>')
        i = 0
        for part in decoded_reply.get_payload():
            cid = part.get('content-Id', '').lstrip('<').rstrip('>')
            if (not start or start == cid) and 'start' not in parts:
                parts['start'] = part.get_payload()
            else:
                parts[cid or 'Attachment%d' % i] = part.get_payload()
            i += 1
        return parts

    def exception_handling(self, messages):
        message_labels = []
        for message in messages:
            if message.messageContent:
                message_labels.append({
                    'id': message.id,
                    'message': unicode(message.messageContent),
                })
        log.debug('message: %s' % message_labels)
        return message_labels



# -*- coding: utf-8 -*-
"""Implementation of Dummy Api."""
from roulier.api import Api

DUMMY_LABEL_FORMAT = (
    'ZPL',
)


class DummyApi(Api):
    def _service(self):
        schema = super(DummyApi, self)._service()
        schema['labelFormat']['allowed'] = DUMMY_LABEL_FORMAT
        schema['labelFormat']['default'] = 'ZPL'
        return schema

    def _to_address(self):
        schema = super(DummyApi, self)._to_address()
        schema['dept'] = {'default': '', 'description': 'Region code'}
        return schema

    def _parcel(self):
        schema = super(DummyApi, self)._parcel()
        schema['reference'] = {'default': '', 'description': 'Parcel reference'}
        return schema

    def _auth(self):
        schema = super(DummyApi, self)._auth()
        schema['login'].update({'required': False})
        schema['password'].update({'required': False})
        return schema



# -*- coding: utf-8 -*-
"""Transform input to laposte compatible xml."""
import logging
from jinja2 import Environment, PackageLoader
from roulier.codec import Encoder
from roulier.exception import InvalidApiInput
from .laposte_api import LaposteApi, LAPOSTE_LABEL_FORMAT
LAPOSTE_ACTIONS = ('generateLabelRequest', 'getProductInter')
_logger = logging.getLogger(__name__)


class LaposteEncoder(Encoder):
    """Transform input to laposte compatible xml."""

    def encode(self, api_input, action):
        """Transform input to laposte compatible xml."""
        if not (action in LAPOSTE_ACTIONS):
            raise Exception(
                'action %s not in %s' % (action, ', '.join(LAPOSTE_ACTIONS)))

        api = LaposteApi()
        if not api.validate(api_input):
            _logger.warning('Laposte api call exception:')
            raise InvalidApiInput(
                {'api_call_exception': api.errors(api_input)})
        data = api.normalize(api_input)

        data['service']['labelFormat'] = self.lookup_label_format(
            data['service']['labelFormat'])

        env = Environment(
            loader=PackageLoader('roulier', '/carriers/laposte/templates'),
            extensions=['jinja2.ext.with_', 'jinja2.ext.autoescape'],
            autoescape=True)

        template = env.get_template("laposte_%s.xml" % action)
        return {
            "body": template.render(
                service=data['service'],
                parcel=data['parcel'],
                auth=data['auth'],
                sender_address=data['from_address'],
                receiver_address=data['to_address'],
                customs=data['customs']),
            "headers": data['auth']
        }

    def api(self):
        """Return API we are expecting."""
        api = LaposteApi()
        return api.api_values()

    def lookup_label_format(self, label_format="ZPL"):
        """Get a Laposte compatible format of label.

        args:
            label_format: (str) ZPL or ZPL_10x15_300dpi
        return
            a value taken in LAPOSTE_LABEL_FORMAT
        """
        lookup = {
            'ZPL': 'ZPL_10x15_300dpi',
            'DPL': 'DPL_10x15_300dpi',
            'PDF': 'PDF_10x15_300dpi',
        }
        if label_format in LAPOSTE_LABEL_FORMAT:
            return label_format
        return lookup.get(label_format, 'PDF_10x15_300dpi')



# -*- coding: utf-8 -*-
"""Mock class."""
from roulier.codec import Decoder


class DummyDecoder(Decoder):
    """Mock class."""

    def decode(self, zpl_string):
        """Return {}."""
        return {"zpl": zpl_string}



# -*- coding: utf-8 -*-
"""Implementation for Geodis."""
from geodis_encoder import GeodisEncoder
from geodis_decoder import GeodisDecoder
from geodis_transport import GeodisTransport
from roulier.carrier import Carrier


class Geodis(Carrier):
    """Implementation for Geodis."""

    encoder = GeodisEncoder()
    decoder = GeodisDecoder()
    ws = GeodisTransport()

    def api(self):
        """Expose how to communicate with Geodis."""
        return self.encoder.api()

    def get(self, data, action):
        """Run an action with data against Geodis WS."""
        request = self.encoder.encode(data, action)
        response = self.ws.send(request)
        if response['status'] == 'error':
            return response
        parts = response['attachement']
        return self.decoder.decode(response, parts)

    # shortcuts
    def get_label(self, data):
        """Genereate a generateLabelRequest."""
        return self.get(data, 'demandeImpressionEtiquette')



# -*- coding: utf-8 -*-
"""Carrier interface."""
import abc


class Carrier(object):
    """Carrier interface."""

    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def api(self):
        """Expose how to communicate with a carrier implementation."""
        return

    @abc.abstractmethod
    def get(self, action):
        """Invoke an action for a carrier."""
        return



# -*- coding: utf-8 -*-
"""Implementation of Geodis Api."""
from roulier.api import Api

GEODIS_LABEL_FORMAT = (
    'PDF',
    'HTML',
    'EPL',
    'ZPL',
    'XML',
)


class GeodisApi(Api):
    def _service(self):
        schema = super(GeodisApi, self)._service()
        schema['labelFormat']['allowed'] = GEODIS_LABEL_FORMAT
        schema['labelFormat']['default'] = 'ZPL'
        schema['labelFormat'].update({'required': True, 'empty': False})
        schema['product'].update({'required': True, 'empty': False})
        schema['agencyId'].update({'required': True, 'empty': False})
        schema['customerId'].update({'required': True, 'empty': False})
        schema['shippingId'].update({'required': True, 'empty': False})
        schema['is_test'] = {
            'type': 'boolean', 'default': True,
            'description': 'Use test WS'}

        return schema

    def _address(self):
        schema = super(GeodisApi, self)._address()
        schema['country'].update({'required': True, 'empty': False})
        schema['zip'].update({'required': True, 'empty': False})
        schema['city'].update({'required': True, 'empty': False})
        return schema

    def _from_address(self):
        schema = super(GeodisApi, self)._from_address()
        schema['phone'].update({'required': True, 'empty': False})
        schema['street1']['required'] = False
        return schema

    def _auth(self):
        schema = super(GeodisApi, self)._auth()
        schema['login'].update({'required': True, 'empty': False})
        schema['password']['required'] = False
        return schema



"""Utilities for WS."""
from lxml import etree
from jinja2 import Environment, PackageLoader


def remove_empty_tags(xml, ouput_as_string=True):
    """Remove empty tags with xslt transformation.

    param: xml a string or a etree type
    return: unicode string or lxml.etree._XSLTResultTree
    """
    # use Jinja env for getting the path of template file
    # pkg_resouces may be an alternative, but we already
    # have Jinja
    env = Environment(
        loader=PackageLoader('roulier', 'templates'),
        extensions=['jinja2.ext.with_'])
    template = env.get_template("remove_empty_tags.xsl")
    xsl = etree.parse(open(template.filename))
    transform = etree.XSLT(xsl)

    if isinstance(xml, basestring):
        xml = etree.fromstring(xml)
    # else we asume xml is an lxml.etree
    if ouput_as_string:
        return unicode(transform(xml))
    else:
        return transform(xml)



# -*- coding: utf-8 -*-
"""Laposte XML -> Python."""
from lxml import objectify
from roulier.codec import Decoder


class LaposteDecoder(Decoder):
    """Laposte XML -> Python."""

    def decode(self, response, parts):
        payload_xml = response['payload']
        tag, content = self.decode_payload(payload_xml)
        if tag.endswith('getProductInterResponse'):
            return content
        else:
            # tag is generateLabelResponse
            label_cid = content.get('label').getchildren()[0].attrib['href']
            if content.get('cn23'):
                cn23_cid = content.get('cn23').getchildren()[0].attrib['href']
            else:
                cn23_cid = False
                content['cn23'] = False

            content['label'] = parts[label_cid.replace('cid:', '')]
            if cn23_cid:
                content['cn23'] = parts[cn23_cid.replace('cid:', '')]

        return content

    def decode_payload(self, xml_string):
        """Laposte XML -> Python."""
        def get_product_inter(msg):
            """Understand a getProductInterResponse."""
            x = {
                "product": msg.product,
                "partnerType": msg.partnerType
            }
            return x

        def generate_label_response(msg):
            """Understand a generateLabelResponse."""
            x = {
                "parcelNumber": msg.labelResponse.parcelNumber,
                "parcelNumberPartner": msg.labelResponse.find(
                    'parcelNumberPartner'),
                "cn23": msg.labelResponse.find('cn23'),
                "label": msg.labelResponse.find('label'),
            }
            return x

        xml = objectify.fromstring(xml_string)
        tag = xml.tag
        lookup = {
            "{http://sls.ws.coliposte.fr}getProductInterResponse":
                get_product_inter,
            "{http://sls.ws.coliposte.fr}generateLabelResponse":
                generate_label_response
        }
        return tag, lookup[tag](xml.xpath('//return')[0])



# -*- coding: utf-8 -*-
"""Factory of main classes."""
from .carriers.laposte.laposte import Laposte
from .carriers.dummy.dummy import Dummy
from .carriers.geodis.geodis import Geodis


def _carriers():
    """Get names:class of carriers.

    You may use the factory get('laposte') instead.
    """
    return {
        "laposte": Laposte,
        "dummy": Dummy,
        "geodis": Geodis,
    }


def get_carriers():
    """Get name of available carriers.

    return: list of strings
    """
    return _carriers().keys()


def get(carrier):
    """Get a 1 method carrier implementation.

    If you need more, like only encode or only transport
    (Webservice), instanciate class directly like:
    from roulier.carriers.laposte import LaposteTransport
    ws = LaposteTransport()
    ws.send(data)
    """
    carrier_obj = _carriers().get(carrier.lower())

    if carrier_obj:
        return carrier_obj()
    else:
        raise BaseException("Carrier not found")



# -*- coding: utf-8 -*-
"""Geodis XML -> Python."""
from lxml import objectify
from roulier.codec import Decoder


class GeodisDecoder(Decoder):
    """Geodis XML -> Python."""

    def decode(self, response, parts):
        payload_xml = response['payload']
        tag, content = self.decode_payload(payload_xml, parts)

        return content

    def decode_payload(self, payload, parts):
        """Geodis XML -> Python."""

        def reponse_impression_etiquette(msg, parts):
            x = {
                'barcode': payload.infoColis.cab,
                'cab': payload.cabRouting,
                'label': parts,
            }
            return x

        tag = payload.tag
        lookup = {
            "{http://impression.service.web.etiquette.geodis.com}reponseImpressionEtiquette":
                reponse_impression_etiquette,
        }
        return tag, lookup[tag](payload, parts)



# -*- coding: utf-8 -*-
"""Implementation for Laposte."""

from .laposte_encoder import LaposteEncoder
from .laposte_decoder import LaposteDecoder
from .laposte_transport import LaposteTransport
from roulier.carrier import Carrier


class Laposte(Carrier):
    """Implementation for Laposte."""

    encoder = LaposteEncoder()
    decoder = LaposteDecoder()
    ws = LaposteTransport()

    def api(self):
        """Expose how to communicate with Laposte."""
        return self.encoder.api()

    def get(self, data, action):
        """Run an action with data against Laposte WS."""
        request = self.encoder.encode(data, action)
        response = self.ws.send(request)
        if response.get('message') and response['message']['exception']:
            return {
                'status': 'error',
                'messages': self.ws.exception_handling(
                    response['message']['message']),
                'response': response['response'],
            }
        parts = self.ws.get_parts(response['response'])
        return self.decoder.decode(response, parts)

    # shortcuts
    def get_label(self, data):
        """Genereate a generateLabelRequest."""
        return self.get(data, 'generateLabelRequest')



# -*- coding: utf-8 -*-
"""API interface."""
from cerberus import Validator


class MyValidator(Validator):
    """Custom validator."""

    def _validate_description(self, description, field, value):
        """Allow 'description' in schema.

        The rule's arguments are validated against this schema:
        { 'description': 'a string'}
        """
        pass


class Api(object):
    """Define expected fields of carriers.

    This class should be overriden by each carrier.
    """

    def __init__(self):
        """."""
        v = MyValidator()
        v.allow_unknown = True
        v.purge_unknown = True
        self._validator = v

    def _address(self):
        return {
            'company': {'type': 'string', 'default': '', 'description': 'Company'},
            'name': {'type': 'string', 'default': '', 'required': True, 'empty': False},
            'street1': {'type': 'string', 'default': ''},
            'street2': {'type': 'string', 'default': ''},
            'country': {'type': 'string', 'default': '', 'description': 'ISO 3166-1 alpha-2 '},
            'city': {'type': 'string', 'default': ''},
            'zip': {'type': 'string', 'default': ''},
            'phone': {'type': 'string', 'default': '', 'description': 'Phone'},
            'email': {'type': 'string', 'default': ''},
        }

    def _from_address(self):
        address = self._address()
        return address

    def _to_address(self):
        address = self._address()
        address['street1'].update({'required': True, 'empty': False})
        address['country'].update({'required': True, 'empty': False})
        address['city'].update({'required': True, 'empty': False})
        address['zip'].update({'required': True, 'empty': False})
        return address

    def _parcel(self):
        return {
            "weight": {'type': 'float', 'default': '', 'description': 'Weight in kg', 'required': True, 'empty': False},
        }

    def _service(self):
        return {
            "product": {'default': '', 'description': ''},
            "agencyId": {'default': '', 'description': ''},
            "customerId": {'default': '', 'description': ''},
            "shippingId": {'default': ''},
            'shippingDate': {'default': '', 'type': 'string', 'required': True, 'empty': False, 'description': 'When the carrier has the package. Format: YYYY/MM/DD'},
            'reference1': {'type': 'string', 'default': '', 'description': 'Additionnal info visible by the client. Example : order number'},
            'reference2': {'type': 'string', 'default': ''},
            'reference3': {'type': 'string', 'default': ''},
            "labelFormat": {'description': 'Format of output (usually pdf or zpl)', 'default': ''},
            "instructions": {'description': 'Additionnal instructions for delivery', 'default': ''},
        }

    def _auth(self):
        return {
            'login': {'type': 'string', 'default': ''},
            'password': {'type': 'string', 'default': ''},
        }

    def _schemas(self):
        return {
            'service': self._service(),
            'auth': self._auth(),
            'parcel': self._parcel(),
            'from_address': self._from_address(),
            'to_address': self._to_address(),
        }

    def api_schema(self):
        """Return the expected schema of the api.

        A validation schema is a mapping, usually a dict.
        Schema keys are the keys allowed in the target dictionary.
        Schema values express the rules that must be matched
        by the corresponding target values.

        See http://docs.python-cerberus.org/en/stable/schemas.html

        """
        v = MyValidator()
        schemas = self._schemas()

        return {
            s: {
                'schema': schemas[s],
                'default': v.normalized({}, schemas[s])
            }
            for s in schemas}

    def api_values(self):
        """Return a dict containing expected keys.

        It's a normalized version of the schema.
        """
        return self.normalize({})

    def errors(self, data):
        """Return validation errors."""
        self._validator.validate(data, self.api_schema())
        return self._validator.errors

    def validate(self, data):
        """Ensure the data are valid.

        returns: bool

        See also errors()
        """
        return self._validator.validate(data, self.api_schema())

    def normalize(self, data):
        """Retrurn a normalized dict based on input.

        See http://docs.python-cerberus.org/en/stable/usage.html
        """
        return self._validator.normalized(data, self.api_schema())



# -*- coding: utf-8 -*-
"""Fake implementation of dummy."""
from roulier.transport import Transport


class DummyTransport(Transport):
    """Mock class because we generate ZPL inhouse."""

    STATUS_SUCCESS = "Success"

    def send(self, body):
        """Call this function.

        Args:
            body: ZPL in a string
        Return:
            {
                status: STATUS_SUCCES or STATUS_ERROR, (string)
                message: more info about status of result (None)
                response: (None)
                payload: usefull payload (if success) (body as string)

            }
        """
        return {
            "status": self.STATUS_SUCCESS,
            "message": None,
            "response": None,
            "payload": body,
        }



# -*- coding: utf-8 -*-
from . import carrier
from . import codec
from . import transport
from . import roulier
from . import exception
import logging

__all__ = [roulier]
#__version__ = open('VERSION').read().strip()
log = logging.getLogger(__name__)
log.addHandler(logging.NullHandler())



# -*- coding: utf-8 -*-
"""Send a request to a carrier and get the result."""
import abc


class Transport(object):
    """Send a request to a carrier and get the result."""

    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def send(self, input):
        """Send a request to a carrier and get the result.

        Args:
            input: carrier specific
        Return:
            carrier specific
        """
        return
